// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneComponent.hpp"
#include "event/Event.hpp"

typedef uint32_t entityid_t;
typedef int64_t entitytileposition_t;

#define ENTITY_STEP_SPEED_DEFAULT 3.0f
#define ENTITY_STEP_SPEED_RUNNING 6.0f

namespace Dawn {
  class World;

  enum class EntityDirection {
    Up,
    Down,
    Left,
    Right
  };
  
  struct EntityTilePosition {
    entitytileposition_t x, y, z;
    
    /**
     * Converts the tile position to a world space position.
     * 
     * @return This tile position in world space.
     */
    const glm::vec3 toWorldSpace();
  };

  class Entity : public SceneComponent {
    private:
      struct EntityTilePosition lastTilePosition;
      enum EntityDirection direction = EntityDirection::Down;
      float_t stepTime = 0.0f;
      float_t stepSpeed = ENTITY_STEP_SPEED_DEFAULT;

    protected:
      struct EntityTilePosition tilePosition;

    public:
      std::weak_ptr<World> world;
      Event<> eventStepStart;
      Event<> eventStepEnd;
      Event<> eventMove;
      
      void onInit() override;
      void onDispose() override;
      void move(
        const enum EntityDirection direction,
        const float_t stepSpeed = ENTITY_STEP_SPEED_DEFAULT
      );
      void setPosition(struct EntityTilePosition position);

      bool_t isMoving();
  };
}